/**
 * @file main.c
 * @author Jake G
 * @date 18 September 2024
 * @brief File contains the main function.
 *
 * This file contains the main logic loop and exists to setup the values 
 * specific to the micro-controller. All other functions and configuration are
 * extracted into separate source files and headers for configuration.
 */

#define F_CPU 3333333UL

//This can prevent issues with utils/delay.h library with the gcc toolchain
#define __DELAY_BACKWARD_COMPATIBLE__

#include "config.h"
#include "RegEdit.h"
#include "zero_cross_detection.h"
#include "ADC.h"
#include "EnOut.h"
#include "load.h"
#include "Enable.h"
#include <avr/cpufunc.h> /* Required header file */
#include <avr/io.h>
#include <util/delay.h>



//Set the function pointer for the delay func
void (*Delay_MicroSeconds)(double us) = _delay_us;


int main(int argc, char **argv)
{ 

    Enable_SetPinsLow();
    EnOut_SetupPins();
    ADC_Setup();

    while(true){
        for(int i = 0; i < GatePulsesQty; i++){
            ZCD_Poll();
            _delay_us(Tau);
            EnOut_SetAllHigh();  //Only G1 exists in High power mode
            EnOut_PulsePins(GatePulses[i]);
        }
        
        //The G1 pin is low at this point.
        _delay_ms(2500);
        Enable_SetPinsHigh();
        ZCD_Poll();
        
        while(true){
            //Enable pins are enabled(set high) if the ADCLOAD value is valid.
            _delay_us(Tau);
            Load_HandleLoadPortA(ADC_LOAD1, EN1);
            Load_HandleLoadPortB(ADC_LOAD2, EN2);
            Load_HandleLoadPortB(ADC_LOAD3, EN3);
        }
    }
}
